﻿Imports IdeaBlade.EntityModel
Imports IdeaBlade.Core
Imports IdeaBlade.Validation

Partial Public Class Employee
    Implements IVerifierProvider

#Region "Verification"

#Region "GetVerifiers"
    Public Function GetVerifiers(ByVal pVerifierProviderContext As Object) As IEnumerable(Of Verifier) Implements IVerifierProvider.GetVerifiers

        Dim verifiers As New List(Of Verifier)()

        verifiers.Add(GetHireDateRangeVerifier())
        verifiers.Add(New BirthDateRangeVerifier())
        verifiers.Add(GetBornBeforeHiredVerifier())
        verifiers.Add(GetPhoneNumberVerifier(Employee.PropertyMetadata.HomePhone))

        Return verifiers
    End Function
#End Region

#Region "Hire Date Verifier"
    ''' <summary>Get a GetHireDateRangeVerifier.</summary>
    ''' <remarks>
    ''' Demonstrates building a highly focused verifier
    ''' by encapsulation a standard verifier
    ''' and its configuration.
    ''' </remarks>
    Private Shared Function GetHireDateRangeVerifier() As Verifier
        Dim v As Verifier = New DateTimeRangeVerifier(GetType(Employee), Employee.PropertyMetadata.HireDate.Name, False, MinHireDate, True, MaxHireDate, False) ' ending max date (exclusive) -  starting min date (inclusive) -  Non-null value is not required -  Property trigger -  Type of the object being verified
        Return v
    End Function

    Private Shared ReadOnly Property MinHireDate() As Date
        Get
            Return New Date(1990, 1, 1)
        End Get
    End Property
    Private Shared ReadOnly Property MaxHireDate() As Date
        Get
            Return Date.Today.AddMonths(1)
        End Get
    End Property

#End Region

#Region "BirthDateRangeVerifier inner class"

    ''' <summary>Get the minimum BirthDate allowed.</summary>
    Private Shared ReadOnly Property MinBirthDate() As Date
        Get
            Return New Date(1900, 1, 1)
        End Get
    End Property

    ''' <summary>BirthDate Range Verifier</summary>
    ''' <remarks>
    ''' Illustrates changing the error messaging for a particular property.
    ''' Have to subclass to take control of the messaging.
    ''' Here the message is statically known so we override
    ''' <see cref="M:Description"/>;
    ''' if it were dynamic or if
    ''' <see cref="T:DateTimeRangeVerifier"/> constructed the
    ''' message dynamically, we would have overridden
    ''' <see cref="M:VerifyValue"/> and manipulated the
    ''' message while creating the <see cref="T:VerifierResult"/>.
    ''' </remarks>
    Private Class BirthDateRangeVerifier
        Inherits DateTimeRangeVerifier
        ''' <summary>Default Ctor,</summary>
        ''' <remarks>
        ''' BirthDate is not required,
        ''' must be on or after global min date (<see cref="M:MinBirthDate"/>), 
        ''' and before today.
        ''' </remarks>
        Public Sub New() ' starting min date (inclusive) -  Non-null value is not required -  Property trigger -  Type of the object being verified
            MyBase.New(GetType(Employee), Employee.PropertyMetadata.BirthDate.Name, False, MinBirthDate, True, Date.Today, False)
            Me.VerifierArgs.ErrorMessageInfo.ErrorMessage = "Must be born after " & MinBirthDate.Year.ToString() & " and no later than today: no time travellers allowed!"
        End Sub ' ending max date (exclusive)
    End Class
#End Region

#Region "Born Before Hired Verifier"

    ''' <summary>Get a BornBeforeHiredVerifier.</summary>
    ''' <remarks>
    ''' Demonstrates comparing two property values
    ''' by creating an instance of a 
    ''' <see cref="T:DelegateVerifier{TVerifiedObject}"/>.
    ''' </remarks>
    Private Shared Function GetBornBeforeHiredVerifier() As Verifier
        ' ToDo: localize description
        Dim description As String = "Must be born before hired."
        Dim v As New DelegateVerifier(Of Employee)(description, AddressOf BornBeforeHiredCondition)

        v.AddTriggers(Employee.PropertyMetadata.BirthDate.Name, Employee.PropertyMetadata.HireDate.Name)

        v.VerifierOptions.ExecutionModes = VerifierExecutionModes.InstanceAndOnAfterSetTriggers
        Return v
    End Function

    ''' <summary>
    ''' The <see cref="T:VerifierDelegate{TVerifiedObject}"/>
    ''' for the <see cref="M:GetBornBeforeHiredVerifier"/>.
    ''' </summary>
    Private Shared Function BornBeforeHiredCondition(ByVal pEmp As Employee, ByVal pTriggerContext As TriggerContext, ByVal pVerifierContext As VerifierContext) As VerifierResult

        ' We are not checking the proposed value because don't expect to call it preset
        If pTriggerContext IsNot Nothing AndAlso pTriggerContext.Timing = TriggerTiming.BeforeSet Then
            Throw New VerifierException("BornBeforeHired verifier not implemented for BeforeSet")
        End If
        If (pEmp.BirthDate < pEmp.HireDate) Then
            Return New VerifierResult(True)
        Else
            Return New VerifierResult(False)
        End If
    End Function

#End Region

#Region "Phone Number Verifier"
    ''' <summary>Get a GetPhoneNumberVerifier.</summary>
    ''' <remarks>
    ''' Encapsulates a standard RegexVerifier, subclassed so the description can be customized.
    ''' </remarks>
    Private Shared Function GetPhoneNumberVerifier(ByVal pPhoneEntityProperty As EntityProperty) As Verifier
        Return New PhoneNumberVerifier(pPhoneEntityProperty.EntityType, pPhoneEntityProperty.Name, False, NamedRegexPattern.USPhone) ' Regex pattern to use -  Non-null value is not required -  Trigger -  Type of object being verified
    End Function

    Private Class PhoneNumberVerifier
        Inherits RegexVerifier
        Public Sub New(ByVal pApplicableType As Type, ByVal pPropertyName As String, ByVal IsRequired As Boolean, ByVal pattern As NamedRegexPattern)
            MyBase.New(pApplicableType, pPropertyName, IsRequired, pattern)
            Me.VerifierArgs.ErrorMessageInfo.ErrorMessage = " including area code [e.g., (206)555-1212, 206-555-1212, or 206.555.1212]."
        End Sub


    End Class

#End Region

#End Region

End Class
